.filer-pane-divider {
  border: 1px solid #C8C8C8 !important;
  /**background-color: #F0F0F0 !important;*/
  border-radius: 1px;
  /*left: 50%;*/
}
/** viewer表示時の行番号スタイル */
pre{
  counter-reset: rowNumber;
}
pre span.row-number{
  counter-increment: rowNumber;
}
pre span.row-number::before { 
  content:counter(rowNumber);
  /* 以下少しだけ装飾 */
  width: 2rem;
  display: inline-block;
  color: #aaa;
}
/** 選択状態のスタイル */
.card-hover:hover {
  box-shadow: 0 0 8px gray;
}
.card-selected {
  box-shadow: 0 0 8px red;
}
.pointer {
  cursor: pointer;
}
